from typing import Any

from django.contrib.admin.views.main import ChangeList
from django.http import HttpRequest
from django.template import Library
from django.utils.safestring import SafeString

register: Library
CHECKBOX_TMPL: str

def result_tree(context: dict[str, Any], cl: ChangeList, request: HttpRequest) -> SafeString: ...



from typing import Any, ClassVar, Self

from django.db import models
from django.db.models import QuerySet
from treebeard.models import Node
from treebeard.numconv import NumConv

class MP_NodeQuerySet(models.QuerySet["MP_Node"]):
    def delete(self, *args: Any, **kwargs: Any) -> tuple[int, dict[str, int]]: ...

class MP_NodeManager(models.Manager["MP_Node"]):
    def get_queryset(self) -> MP_NodeQuerySet: ...

class MP_ComplexAddMoveHandler:
    node_cls: type[MP_Node]
    def increment_numchild(self, path: str) -> None: ...
    def decrement_numchild(self, path: str) -> None: ...
    def reorder_nodes_before_add_or_move(
        self,
        pos: str,
        newpos: int | None,
        newdepth: int,
        target: MP_Node,
        siblings: QuerySet[MP_Node] | list[MP_Node],
        oldpath: str | None = ...,
        movebranch: bool = ...,
    ) -> tuple[str | None, str]: ...
    def set_newpath_in_branches(self, oldpath: str, newpath: str) -> None: ...

class MP_AddRootHandler:
    cls: type[MP_Node]
    kwargs: dict[str, Any]
    def __init__(self, cls: type[MP_Node], **kwargs: Any) -> None: ...
    def process(self) -> MP_Node: ...

class MP_AddChildHandler:
    node: MP_Node
    node_cls: type[MP_Node]
    kwargs: dict[str, Any]
    def __init__(self, node: MP_Node, creation_kwargs: dict[str, Any]) -> None: ...
    def process(self) -> MP_Node: ...

class MP_AddSiblingHandler(MP_ComplexAddMoveHandler):
    node: MP_Node
    node_cls: type[MP_Node]
    pos: str
    kwargs: dict[str, Any]
    def __init__(self, node: MP_Node, pos: str | None, creation_kwargs: dict[str, Any]) -> None: ...
    def process(self) -> MP_Node: ...

class MP_MoveHandler(MP_ComplexAddMoveHandler):
    node: MP_Node
    node_cls: type[MP_Node]
    target: MP_Node
    pos: str | None
    def __init__(self, node: MP_Node, target: MP_Node, pos: str | None = ...) -> None: ...
    def process(self) -> None: ...
    def update_parent_counts_after_move(self, oldpath: str, newpath: str) -> None: ...
    def update_move_to_child_vars(self) -> tuple[int, list[MP_Node] | QuerySet[MP_Node], int | None]: ...

class MP_Node(Node):
    steplen: int
    alphabet: str
    node_order_by: list[str]
    path: models.CharField[str, str]
    depth: models.PositiveIntegerField[int, int]
    numchild: models.PositiveIntegerField[int, int]

    TREEBEARD_IDENTIFYING_FIELD: str
    MOVENODE_FORM_EXCLUDED_FIELDS: tuple[str, ...]

    objects: ClassVar[MP_NodeManager]  # type: ignore[assignment]

    numconv_obj_: NumConv | None

    @classmethod
    def numconv_obj(cls) -> NumConv: ...
    @classmethod
    def add_root(cls, **kwargs: Any) -> Self: ...
    @classmethod
    def dump_bulk(cls, parent: Self | None = ..., keep_ids: bool = ...) -> list[dict[str, Any]]: ...
    @classmethod
    def find_problems(cls) -> tuple[list[int], list[int], list[int], list[int], list[int]]: ...
    @classmethod
    def fix_tree(cls, fix_paths: bool = ..., parent: Self | None = ...) -> None: ...
    @classmethod
    def get_tree(cls, parent: Self | None = ...) -> QuerySet[Self]: ...
    @classmethod
    def get_root_nodes(cls) -> QuerySet[Self]: ...
    @classmethod
    def get_descendants_group_count(cls, parent: Self | None = ...) -> QuerySet[Self]: ...
    def get_depth(self) -> int: ...
    def get_siblings(self) -> QuerySet[Self]: ...
    def get_children(self) -> QuerySet[Self]: ...
    def get_next_sibling(self) -> Self | None: ...
    def get_descendants(self, include_self: bool = ...) -> QuerySet[Self]: ...
    def get_prev_sibling(self) -> Self | None: ...
    def get_children_count(self) -> int: ...
    def is_sibling_of(self, node: Self) -> bool: ...
    def is_child_of(self, node: Self) -> bool: ...
    def is_descendant_of(self, node: Self) -> bool: ...
    def add_child(self, **kwargs: Any) -> Self: ...
    def add_sibling(self, pos: str | None = ..., **kwargs: Any) -> Self: ...
    def get_root(self) -> Self: ...
    def is_root(self) -> bool: ...
    def is_leaf(self) -> bool: ...
    def get_ancestors(self) -> QuerySet[Self]: ...
    def get_parent(self, update: bool = ...) -> Self | None: ...
    def move(self, target: Self, pos: str | None = ...) -> None: ...

    class Meta:
        abstract: bool



SECRET_KEY = "test-secret-key"

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.auth",
    "django.contrib.admin",
    "treebeard",
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}



from collections.abc import Iterable
from typing import Any, ClassVar, Self

from django.db import models
from django.db.models import QuerySet
from treebeard.ltree.fields import PathField as PathField
from treebeard.ltree.fields import PathValue as PathValue
from treebeard.models import Node

class InvalidLabelConstraints(Exception): ...

def generate_label(
    skip: set[str],
    before: str | None = ...,
    after: str | None = ...,
) -> str: ...
def generate_path(
    prefix: PathValue | None = ...,
    before: str | None = ...,
    after: str | None = ...,
    skip: Iterable[PathValue] | None = ...,
) -> PathValue: ...

class LT_NodeQuerySet(models.QuerySet["LT_Node"]):
    def delete(self, *args: Any, **kwargs: Any) -> tuple[int, dict[str, int]]: ...

class LT_NodeManager(models.Manager["LT_Node"]):
    def get_queryset(self) -> LT_NodeQuerySet: ...

class LT_ComplexAddMoveHandler: ...

class LT_AddRootHandler:
    cls: type[LT_Node]
    kwargs: dict[str, Any]
    def __init__(self, cls: type[LT_Node], **kwargs: Any) -> None: ...
    def process(self) -> LT_Node: ...

class LT_AddChildHandler:
    node: LT_Node
    node_cls: type[LT_Node]
    kwargs: dict[str, Any]
    def __init__(self, node: LT_Node, creation_kwargs: dict[str, Any]) -> None: ...
    def process(self) -> LT_Node: ...

class LT_AddSiblingHandler(LT_ComplexAddMoveHandler):
    node: LT_Node
    node_cls: type[LT_Node]
    pos: str
    kwargs: dict[str, Any]
    def __init__(self, node: LT_Node, pos: str | None, creation_kwargs: dict[str, Any]) -> None: ...
    def process(self) -> LT_Node: ...

class LT_MoveHandler(LT_ComplexAddMoveHandler):
    node: LT_Node
    node_cls: type[LT_Node]
    target: LT_Node
    pos: str | None
    def __init__(self, node: LT_Node, target: LT_Node, pos: str | None = ...) -> None: ...
    def process(self) -> None: ...

class LT_Node(Node):
    node_order_by: list[str]
    path: PathField

    TREEBEARD_IDENTIFYING_FIELD: str
    MOVENODE_FORM_EXCLUDED_FIELDS: tuple[str, ...]

    objects: ClassVar[LT_NodeManager]  # type: ignore[assignment]

    @classmethod
    def add_root(cls, **kwargs: Any) -> Self: ...
    @classmethod
    def dump_bulk(cls, parent: Self | None = ..., keep_ids: bool = ...) -> list[dict[str, Any]]: ...
    @classmethod
    def get_tree(cls, parent: Self | None = ...) -> QuerySet[Self]: ...
    @classmethod
    def get_root_nodes(cls) -> QuerySet[Self]: ...
    @classmethod
    def get_descendants_group_count(cls, parent: Self | None = ...) -> QuerySet[Self]: ...
    def get_depth(self) -> int: ...
    def get_siblings(self) -> QuerySet[Self]: ...
    def get_children(self) -> QuerySet[Self]: ...
    def get_next_sibling(self) -> Self | None: ...
    def get_descendants(self, include_self: bool = ...) -> QuerySet[Self]: ...
    def get_prev_sibling(self) -> Self | None: ...
    def get_children_count(self) -> int: ...
    def is_sibling_of(self, node: Self) -> bool: ...
    def is_child_of(self, node: Self) -> bool: ...
    def is_descendant_of(self, node: Self) -> bool: ...
    def add_child(self, **kwargs: Any) -> Self: ...
    def add_sibling(self, pos: str | None = ..., **kwargs: Any) -> Self: ...
    def get_root(self) -> Self: ...
    def is_root(self) -> bool: ...
    def is_leaf(self) -> bool: ...
    def get_ancestors(self) -> QuerySet[Self]: ...
    def get_parent(self, update: bool = ...) -> Self | None: ...
    def move(self, target: Self, pos: str | None = ...) -> None: ...

    class Meta:
        abstract: bool



"""Run stubtest with Django configured."""

import os
import sys

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "tests.mypy_test_settings")

import django

django.setup()

from mypy.stubtest import main  # noqa: E402

sys.exit(main())



from typing import Any

from django import forms
from django.utils.safestring import SafeString
from treebeard.models import Node

class TreeNodeChoiceField(forms.ModelChoiceField):
    DEPTH_SEPARATOR: SafeString
    def label_from_instance(self, obj: Node) -> SafeString: ...

class MoveNodeForm(forms.ModelForm):
    treebeard_position: forms.ChoiceField
    treebeard_ref_node: TreeNodeChoiceField
    is_sorted: bool | list[str] | None
    def __init__(
        self, *args: Any, initial: dict[str, Any] | None = ..., instance: Node | None = ..., **kwargs: Any
    ) -> None: ...
    def save(self, commit: bool = ...) -> Node: ...

def movenodeform_factory(
    model: type[Node],
    form: type[MoveNodeForm] = ...,
    exclude: tuple[str, ...] | None = ...,
    **kwargs: Any,
) -> type[MoveNodeForm]: ...



from typing import Any, ClassVar, Self

from django.db import models
from django.db.models import QuerySet
from treebeard.models import Node

class AL_NodeManager(models.Manager["AL_Node"]):
    def get_queryset(self) -> QuerySet[AL_Node]: ...

class AL_Node(Node):
    objects: ClassVar[AL_NodeManager]  # type: ignore[assignment]
    node_order_by: list[str] | None

    TREEBEARD_IDENTIFYING_FIELD: str
    MOVENODE_FORM_EXCLUDED_FIELDS: tuple[str, ...]

    parent: models.ForeignKey[Self | None, Self | None]
    sib_order: models.PositiveIntegerField[int, int]

    @classmethod
    def add_root(cls, **kwargs: Any) -> Self: ...
    @classmethod
    def get_root_nodes(cls) -> QuerySet[Self]: ...
    def get_depth(self, update: bool = ...) -> int: ...
    def get_children(self) -> QuerySet[Self]: ...
    def get_parent(self, update: bool = ...) -> Self | None: ...
    def get_ancestors(self) -> list[Self]: ...
    def get_root(self) -> Self: ...
    def is_descendant_of(self, node: Self) -> bool: ...
    @classmethod
    def dump_bulk(cls, parent: Self | None = ..., keep_ids: bool = ...) -> list[dict[str, Any]]: ...
    def add_child(self, **kwargs: Any) -> Self: ...
    @classmethod
    def get_tree(cls, parent: Self | None = ...) -> list[Self]: ...
    def get_descendants(self, include_self: bool = ...) -> list[Self]: ...
    def get_descendant_count(self) -> int: ...
    def get_siblings(self) -> QuerySet[Self]: ...
    def get_prev_sibling(self) -> Self | None: ...
    def get_next_sibling(self) -> Self | None: ...
    def add_sibling(self, pos: str | None = ..., **kwargs: Any) -> Self: ...
    def move(self, target: Self, pos: str | None = ...) -> None: ...

    class Meta:
        abstract: bool



from collections import UserList
from collections.abc import Iterable
from typing import Any

from django import forms
from django.core.validators import RegexValidator
from django.db.backends.base.base import BaseDatabaseWrapper
from django.db.models import IntegerField, Model
from django.db.models.expressions import BaseExpression, Combinable, Func
from django.db.models.fields import CharField, TextField
from django.db.models.lookups import PostgresOperatorLookup, Transform

path_label_validator: RegexValidator

class PathValue(UserList[str]):
    def __init__(self, value: str | Iterable[str]) -> None: ...

class PathValueProxy:
    field_name: str
    def __init__(self, field_name: str) -> None: ...
    def __get__(self, instance: Model | None, owner: type[Model]) -> PathValue | None: ...
    def __set__(self, instance: Model | None, value: str | PathValue | Iterable[str] | None) -> None: ...

class PathFormField(forms.CharField): ...

class PathField(TextField):
    def db_type(self, connection: BaseDatabaseWrapper) -> str: ...
    def contribute_to_class(self, cls: type[Model], name: str, private_only: bool = ...) -> None: ...
    def from_db_value(
        self, value: str | None, expression: BaseExpression, connection: BaseDatabaseWrapper, *args: Any
    ) -> PathValue | None: ...
    def get_prep_value(self, value: str | PathValue | None) -> str | None: ...
    def to_python(self, value: str | PathValue | None) -> PathValue | None: ...
    def get_db_prep_value(
        self, value: str | PathValue | list[str] | None, connection: BaseDatabaseWrapper, prepared: bool = ...
    ) -> str | None: ...

class AncestorLookup(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class DescendantLookup(PostgresOperatorLookup):
    lookup_name: str
    postgres_operator: str

class Subpath(Func):
    function: str
    output_field: PathField
    def __init__(
        self, expression: Combinable | str, pos: int | Combinable, length: int | Combinable | None = ..., **extra: Any
    ) -> None: ...

class Ltree2Text(Transform):
    function: str
    lookup_name: str
    output_field: CharField  # type: ignore[assignment]

class Text2LTree(Transform):
    function: str
    lookup_name: str
    output_field: PathField  # type: ignore[assignment]

class NLevel(Transform):
    lookup_name: str
    function: str
    output_field: IntegerField  # type: ignore[assignment]



from django.contrib.admin.views.main import ChangeList
from django.template import Library

register: Library

def result_tree(cl: ChangeList) -> dict[str, list[dict[str, int | str | None]] | list | ChangeList | int]: ...
def tree_context(cl: ChangeList) -> list[dict[str, str | int]]: ...



BASE85: str

class NumConv:
    radix: int
    alphabet: str
    cached_map: dict[str, int]
    def __init__(self, alphabet: str = ...) -> None: ...
    def int2str(self, num: int) -> str: ...
    def str2int(self, chars: str) -> int: ...



from collections.abc import Iterable
from typing import Any, Self

from django.db import models
from django.db.models import QuerySet

class Node(models.Model):
    TREEBEARD_IDENTIFYING_FIELD: str | None
    MOVENODE_FORM_EXCLUDED_FIELDS: tuple[str, ...]
    node_order_by: list[str] | tuple[str, ...] | None

    @classmethod
    def add_root(cls, **kwargs: Any) -> Self: ...
    @classmethod
    def load_bulk(
        cls, bulk_data: list[dict[str, Any]], parent: Self | None = ..., keep_ids: bool = ...
    ) -> list[int]: ...
    @classmethod
    def dump_bulk(cls, parent: Self | None = ..., keep_ids: bool = ...) -> list[dict[str, Any]]: ...
    @classmethod
    def get_root_nodes(cls) -> QuerySet[Self]: ...
    @classmethod
    def get_first_root_node(cls) -> Self | None: ...
    @classmethod
    def get_last_root_node(cls) -> Self | None: ...
    @classmethod
    def find_problems(cls) -> tuple[list[int], list[int], list[int], list[int], list[int]]: ...
    @classmethod
    def fix_tree(cls) -> None: ...
    @classmethod
    def get_tree(cls, parent: Self | None = ...) -> QuerySet[Self] | list[Self]: ...
    @classmethod
    def get_descendants_group_count(cls, parent: Self | None = ...) -> QuerySet[Self] | list[Self]: ...
    def get_depth(self) -> int: ...
    def get_siblings(self) -> QuerySet[Self]: ...
    def get_children(self) -> QuerySet[Self]: ...
    def get_children_count(self) -> int: ...
    def get_descendants(self) -> QuerySet[Self] | list[Self]: ...
    def get_descendant_count(self) -> int: ...
    def get_first_child(self) -> Self | None: ...
    def get_last_child(self) -> Self | None: ...
    def get_first_sibling(self) -> Self | None: ...
    def get_last_sibling(self) -> Self | None: ...
    def get_prev_sibling(self) -> Self | None: ...
    def get_next_sibling(self) -> Self | None: ...
    def is_sibling_of(self, node: Self) -> bool: ...
    def is_child_of(self, node: Self) -> bool: ...
    def is_descendant_of(self, node: Self) -> bool: ...
    def add_child(self, **kwargs: Any) -> Self: ...
    def add_sibling(self, pos: str | None = ..., **kwargs: Any) -> Self: ...
    def get_root(self) -> Self: ...
    def is_root(self) -> bool: ...
    def is_leaf(self) -> bool: ...
    def get_ancestors(self) -> QuerySet[Self] | list[Self]: ...
    def get_parent(self, update: bool = ...) -> Self | None: ...
    def move(self, target: Self, pos: str | None = ...) -> None: ...
    def delete(self, *args: Any, **kwargs: Any) -> tuple[int, dict[str, int]]: ...
    def get_sorted_pos_queryset(self, siblings: QuerySet[Self], newobj: Self) -> QuerySet[Self]: ...
    def refresh_from_db(
        self,
        using: str | None = ...,
        fields: Iterable[str] | None = ...,
        from_queryset: QuerySet[Self] | None = ...,
    ) -> None: ...
    @classmethod
    def get_annotated_list_qs(cls, qs: QuerySet[Self]) -> list[tuple[Self, dict[str, Any]]]: ...
    @classmethod
    def get_annotated_list(
        cls, parent: Self | None = ..., max_depth: int | None = ...
    ) -> list[tuple[Self, dict[str, Any]]]: ...
    @classmethod
    def tree_model(cls) -> type[Self]: ...

    class Meta:
        abstract: bool



from typing import Any, ClassVar, Self

from django.db import models
from django.db.models import QuerySet
from treebeard.models import Node

def merge_deleted_counters(
    c1: tuple[int, dict[str, int]], c2: tuple[int, dict[str, int]]
) -> tuple[int, dict[str, int]]: ...

class NS_NodeQuerySet(models.QuerySet["NS_Node"]):
    def delete(
        self,
        *args: Any,
        removed_ranges: list[tuple[int, int, int]] | None = ...,
        deleted_counter: tuple[int, dict[str, int]] | None = ...,
        **kwargs: Any,
    ) -> tuple[int, dict[str, int]]: ...

class NS_NodeManager(models.Manager["NS_Node"]):
    def get_queryset(self) -> NS_NodeQuerySet: ...

class NS_Node(Node):
    node_order_by: list[str]
    lft: models.PositiveIntegerField[int, int]
    rgt: models.PositiveIntegerField[int, int]
    tree_id: models.PositiveIntegerField[int, int]
    depth: models.PositiveIntegerField[int, int]

    TREEBEARD_IDENTIFYING_FIELD: str
    MOVENODE_FORM_EXCLUDED_FIELDS: tuple[str, ...]

    objects: ClassVar[NS_NodeManager]  # type: ignore[assignment]

    @classmethod
    def add_root(cls, **kwargs: Any) -> Self: ...
    def add_child(self, **kwargs: Any) -> Self: ...
    def add_sibling(self, pos: str | None = ..., **kwargs: Any) -> Self: ...
    def move(self, target: Self, pos: str | None = ...) -> None: ...
    def get_children(self) -> QuerySet[Self]: ...
    def get_depth(self) -> int: ...
    def is_leaf(self) -> bool: ...
    def get_root(self) -> Self: ...
    def is_root(self) -> bool: ...
    def get_siblings(self) -> QuerySet[Self]: ...
    @classmethod
    def dump_bulk(cls, parent: Self | None = ..., keep_ids: bool = ...) -> list[dict[str, Any]]: ...
    @classmethod
    def get_tree(cls, parent: Self | None = ...) -> QuerySet[Self]: ...
    def get_descendants(self, include_self: bool = ...) -> QuerySet[Self]: ...
    def get_descendant_count(self) -> int: ...  # Runtime returns float but base class declares int; keeping int for LSP
    def get_ancestors(self) -> QuerySet[Self]: ...
    def is_descendant_of(self, node: Self) -> bool: ...
    def get_parent(self, update: bool = ...) -> Self | None: ...
    @classmethod
    def get_root_nodes(cls) -> QuerySet[Self]: ...

    class Meta:
        abstract: bool



from typing import Any, ClassVar

from django.contrib import admin
from django.db.models import QuerySet
from django.http import HttpRequest, HttpResponse, HttpResponseBadRequest
from django.urls.resolvers import URLPattern
from treebeard.forms import MoveNodeForm
from treebeard.models import Node

def check_empty_dict(GET_dict: dict[str, str]) -> bool: ...

class TreeAdmin(admin.ModelAdmin[Node]):
    change_list_template: ClassVar[str]  # type: ignore[assignment]
    def get_queryset(self, request: HttpRequest) -> QuerySet[Node]: ...
    def changelist_view(self, request: HttpRequest, extra_context: dict[str, Any] | None = ...) -> HttpResponse: ...
    def get_urls(self) -> list[URLPattern]: ...
    def get_node(self, node_id: str) -> Node: ...
    def try_to_move_node(
        self, as_child: bool, node: Node, pos: str, request: HttpRequest, target: Node
    ) -> HttpResponse | HttpResponseBadRequest: ...
    def move_node(self, request: HttpRequest) -> HttpResponse | HttpResponseBadRequest: ...

def admin_factory(form_class: type[MoveNodeForm]) -> type[TreeAdmin]: ...
